package us.lakora.bombdrop;

import java.awt.Color;

public class BlockGroup {
	
	public static final int I = 0;
	public static final int O = 1;
	public static final int T = 2;
	public static final int S = 3;
	public static final int Z = 4;
	public static final int J = 5;
	public static final int L = 6;
	public static final int TWO = 7;
	public static final int THREE = 8;

	private int type;
	private Color[][] blocks;
	private Color color;
	private int row;
	private int col;
	public boolean waiting; // extra pause before block locks
	
	/**
	 * The small bomb color.
	 */
	public static final Color SMALL_BOMB = new Color(255, 213, 0);
	
	/**
	 * The big bomb color.
	 */
	public static final Color BIG_BOMB = new Color(255, 149, 0);
	
	public static final Color[] FIGURE_COLORS = {
//		new Color(49, 199, 239),
//		new Color(247, 211, 8),
//		new Color(173, 77, 156),
//		Color.green,
//		Color.red,
//		Color.blue,
//		new Color(239, 121, 33),
//		Color.white,
//		Color.white};
		Color.lightGray,
		Color.lightGray,
		Color.lightGray,
		Color.lightGray,
		Color.lightGray,
		Color.lightGray,
		Color.lightGray,
		Color.lightGray,
		Color.lightGray};
	
	public BlockGroup(int cols) {
		row = 0;
		col = cols/2 - 1;
		type = (int)(Math.random()*9);
		color = FIGURE_COLORS[type];
		Color[] t = {color, color, color, color};
		t[(int)(4*Math.random())] = SMALL_BOMB;
		if (type == I) {
			Color[] l = {color, color, color, color};
			l[(int)(2*Math.random())] = SMALL_BOMB; // to ensure the small bomb is on one of the ends
			Color[][] tblocks = {
					{null,null,null,null},
					{l[0],l[3],l[2],l[1]},
					{null,null,null,null},
					{null,null,null,null},
			};blocks=tblocks;
		} else if (type == O) {
			Color[][] tblocks = {
					{t[0],t[1]},
					{t[2],t[3]},
			}; blocks=tblocks;
		} else if (type == T) {
			Color[][] tblocks = {
					{null,t[0],null},
					{t[1],t[2],t[3]},
					{null,null,null},
			}; blocks=tblocks;
		} else if (type == S) {
//			color = Color.green;
			Color[][] tblocks = {
					{null,t[0],t[1]},
					{t[2],t[3],null},
					{null,null,null},
			}; blocks=tblocks;
		} else if (type == Z) {
//			color = Color.red;
			Color[][] tblocks = {
					{t[0],t[1],null},
					{null,t[2],t[3]},
					{null,null,null},
			}; blocks=tblocks;
		} else if (type == J) {
//			color = Color.blue;
			Color[][] tblocks = {
					{t[0],null,null},
					{t[1],t[2],t[3]},
					{null,null,null},
			}; blocks=tblocks;
		} else if (type == L) {
//			color = new Color(239, 121, 33);
			Color[][] tblocks = {
					{null,null,t[0]},
					{t[1],t[2],t[3]},
					{null,null,null},
			}; blocks=tblocks;
		} else if (type == TWO) {
			Color[] l = {color, color};
			l[(int)(2*Math.random())] = SMALL_BOMB;
			Color[][] tblocks = {
					{null,l[0],null},
					{null,l[1],null},
					{null,null,null},
			}; blocks=tblocks;
		} else if (type == THREE) {
			Color[] l = {color, color, color};
			l[(int)(3*Math.random())] = SMALL_BOMB;
			Color[][] tblocks = {
					{null,null,null},
					{l[0],l[1],l[2]},
					{null,null,null},
			}; blocks=tblocks;
		}
	}

	public Color getColor() {
		return color;
	}
	public int getRow() {
		return row;
	}
	public int getCol() {
		return col;
	}
	public Color[][] getBlocks() {
		return blocks;
	}
	public synchronized void drop() {
		row++;
	}
	public synchronized void undoDrop() {
		row--;
	}
	public synchronized void moveLeft() {
		col--;
	}
	public synchronized void moveRight() {
		col++;
	}
	public synchronized void rotateLeft() {
		if (blocks.length == 2) {
			//make a new array
			Color[][] tblocks = {
					{blocks[0][1], blocks[1][1]},
					{blocks[0][0], blocks[1][0]}};
			//assign to blocks
			blocks = tblocks;
		} else if (blocks.length == 3) {
			//make a new array
			Color[][] tblocks = {
					{blocks[0][2], blocks[1][2], blocks[2][2]},
					{blocks[0][1], blocks[1][1], blocks[2][1]},
					{blocks[0][0], blocks[1][0], blocks[2][0]}};
			//assign to blocks
			blocks = tblocks;
		} else if (blocks.length == 4) {
			//make a new array
			Color[][] tblocks = {
					{blocks[0][3], blocks[1][3], blocks[2][3], blocks[3][3]},
					{blocks[0][2], blocks[1][2], blocks[2][2], blocks[3][2]},
					{blocks[0][1], blocks[1][1], blocks[2][1], blocks[3][1]},
					{blocks[0][0], blocks[1][0], blocks[2][0], blocks[3][0]}};
			//assign to blocks
			blocks = tblocks;
		}
		//the check for collisions is done in PlayingField after rotateLeft is called.
	}
	public synchronized void rotateRight() {
		if (blocks.length == 2) {
			//make a new array
			Color[][] tblocks = {
					{blocks[1][0], blocks[0][0]},
					{blocks[1][1], blocks[0][1]}};
			//assign to blocks
			blocks = tblocks;
		} else if (blocks.length == 3) {
			//make a new array
			Color[][] tblocks = {
					{blocks[2][0], blocks[1][0], blocks[0][0]},
					{blocks[2][1], blocks[1][1], blocks[0][1]},
					{blocks[2][2], blocks[1][2], blocks[0][2]}};
			//assign to blocks
			blocks = tblocks;
		} else if (blocks.length == 4) {
			//make a new array
			Color[][] tblocks = {
					{blocks[3][0], blocks[2][0], blocks[1][0], blocks[0][0]},
					{blocks[3][1], blocks[2][1], blocks[1][1], blocks[0][1]},
					{blocks[3][2], blocks[2][2], blocks[1][2], blocks[0][2]},
					{blocks[3][3], blocks[2][3], blocks[1][3], blocks[0][3]}};
			//assign to blocks
			blocks = tblocks;
		}
		//the check for collisions is done in PlayingField after rotateRight is called.
	}

}
