package us.lakora.bombdrop;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;

import us.lakora.DisplayLicense;

public class BombdropMenu extends JMenuBar {
	private static final long serialVersionUID = 1L;
	private PlayingField pf;
	public static final String TITLE = "Bombdrop 1.1"; // last updated May 13, 2012
	
	public static final String ABOUT_TEXT = 
			"<html><font color=#008000>" + TITLE + "<br/>" +
			"&copy; 2012 Isaac Schemm</font><br/>" +
			"<br/>" +
			"<font color=#800000>Levels 1-30 are from the SFC game \"Super Tetris 2 + Bombliss\" by BPS.</font><br/>" +
			"<br/>" +
			"<font color=#000080>This program is free software: you can redistribute it and/or modify<br/>" +
			"it under the terms of the GNU Affero General Public License as<br/>" +
			"published by the Free Software Foundation, either version 3 of the<br/>" +
			"License, or (at your option) any later version.<br/>" +
			"<br/>" +
			"This program is distributed in the hope that it will be useful,<br/>" +
			"but WITHOUT ANY WARRANTY; without even the implied warranty of<br/>" +
			"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<br/>" +
			"GNU Affero General Public License for more details.<br/>" +
			"<br/>" +
			"You should have received a copy of the GNU Affero General Public License<br/>" +
			"along with this program.  If not, see &lsaquo;http://www.gnu.org/licenses/&rsaquo;.</font><br/>" +
			"<br/>" +
			"<font color=#800000>Because this program is distributed under the terms of the GNU Affero<br/>" +
			"General Public License, the source code must be made available to anyone<br/>" +
			"obtaining the software or to anyone using the software over a network.</font></html>";
	
	public BombdropMenu() {
		JMenu help = new JMenu("Help");
		add(help);
		
		JMenuItem howToPlay = new JMenuItem("How To Play");
		help.add(howToPlay);
		howToPlay.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (pf != null) pf.hardPause(true);
				new HelpDialog().setVisible(true);
				if (pf != null) pf.hardPause(false);
			}
		});
		
		JMenuItem about = new JMenuItem("About");
		help.add(about);
		about.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				showAbout();
			}
		});
	}
	
	private void showAbout() {
		if (pf != null) pf.hardPause(true);
		final JDialog f = new JDialog(null, TITLE, Dialog.DEFAULT_MODALITY_TYPE);
		f.setTitle("About "+TITLE);
		f.setLayout(new BorderLayout());

		JLabel about = new JLabel(ABOUT_TEXT);
		about.setBorder(BorderFactory.createEmptyBorder(8, 0, 8, 8));
		f.add(about, BorderLayout.CENTER);
		
		ImageIcon ii = null;
		URL iconURL = MainWindow.class.getClassLoader().getResource("resources/logo.png");
		if (iconURL != null) {
			ii = new ImageIcon(iconURL);
		}
		
		JLabel icon = new JLabel(ii);
		icon.setVerticalAlignment(JLabel.TOP);
		icon.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
		f.add(icon, BorderLayout.WEST);
		
		JButton showLicense = new JButton("Show License");
		showLicense.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				DisplayLicense.readFileFromJAR(null, "resources/agpl-3.0.txt");
			}
		});
		JButton close = new JButton("Close");
		close.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				f.setVisible(false);
			}
		});
		
		JPanel buttons = new JPanel();
		buttons.add(showLicense);
		buttons.add(close);
		f.add(buttons, BorderLayout.SOUTH);
		
		f.pack();
		f.setVisible(true);
		if (pf != null) pf.hardPause(false);
	}
	
	/**
	 * Used for pause functions
	 * @param pf the field to pause
	 */
	public void setPlayingField(PlayingField pf) {
		this.pf = pf;
	}

}
