package us.lakora.bombdrop;

import java.awt.Color;
import java.awt.Point;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import us.lakora.bombdrop.bombs.BigBomb;
import us.lakora.bombdrop.bombs.Bomb;
import us.lakora.bombdrop.bombs.SmallBomb;


public class ClearThread extends Thread {
	
	private Color[][] blocks;
	private int rows;
	private int cols;
	private PlayingField parent;
	private Score score;
	private Color[] gradient;
	
	private final static Color EXPLOSION_COLOR = new Color(64, 64, 65);
	
	public ClearThread(Color[][] blocks, PlayingField parent, Score score) {
		this.blocks = blocks;
		rows = blocks.length;
		cols = blocks[0].length;
		this.parent = parent;
		this.score = score;
	}
	
	private boolean isEmpty() {
		int i=0, j=0;
		boolean couldBeEmpty = true;
		while (couldBeEmpty && (j < blocks[0].length)) {
			couldBeEmpty = (blocks[i][j] == null);
			i++;
			if (i == blocks.length) {
				i=0; j++;
			}
		}
		return couldBeEmpty;
	}
	
	public void run() {
		if (hasFullLines()) {
			activateFullLines(score, 0);
			if (isEmpty() && (parent.getLevel() > 0)) {
				parent.displayLevelClear = true;
				parent.repaint();
				try {
					Thread.sleep(2000);
				} catch (InterruptedException e) {}
				parent.displayLevelClear = false;
				parent.advanceLevel();
			}
		} else {
			makeBigBombs();
		}
		
		parent.regenerate();
	}
	
    /**
     * Checks if the board contains any full lines.
     *
     * @return true if there are full lines on the board, or
     *         false otherwise
     */
    public boolean hasFullLines() {
        for (int y = rows - 1; y >= 0; y--) {
            if (isLineFull(y)) {
                return true;
            }
        }
        return false;
    }
	
    public boolean isLineFull(int y) {
        if (y < 0 || y >= rows) {
            return true;
        }
        for (int x = 0; x < cols; x++) {
            if (blocks[y][x] == null) {
                return false;
            }
        }
        return true;
    }
	
    /**
     * Scans the board for 2x2 squares, and combines them into a big bomb (makes them cyan).
     */
    public void makeBigBombs() {
    	boolean made = false;
    	if (true) {
    		List<Integer> x_values = new ArrayList<Integer>();
    		List<Integer> y_values = new ArrayList<Integer>();
	    	// Checking for 2x2 squares. These will become cyan-colored (big bombs.)
	        // The last row and column aren't checked - only the corners of 2x2 squares are checked for.
	        for (int y=0; y<rows-1; y++) for (int x=0; x<cols-1; x++) {
	        	Color o = BlockGroup.SMALL_BOMB;
		        if (o.equals(blocks[y][x]) && o.equals(blocks[y+1][x]) && o.equals(blocks[y+1][x+1]) && o.equals(blocks[y][x+1])) {
		        	made = true;
		        	x_values.add(x);
		        	y_values.add(y);
		        }
	        }
	        if (this.gradient == null) {
		        int red1 = BlockGroup.SMALL_BOMB.getRed();
		        int green1 = BlockGroup.SMALL_BOMB.getGreen();
		        int blue1 = BlockGroup.SMALL_BOMB.getBlue();
		        int red2 = BlockGroup.BIG_BOMB.getRed();
		        int green2 = BlockGroup.BIG_BOMB.getGreen();
		        int blue2 = BlockGroup.BIG_BOMB.getBlue();
		        Color transition_25 = new Color(
		        		(3*red1+red2)/4,
		        		(3*green1+green2)/4,
		        		(3*blue1+blue2)/4);
		        Color transition_50 = new Color(
		        		(red1+red2)/2,
		        		(green1+green2)/2,
		        		(blue1+blue2)/2);
		        Color transition_75 = new Color(
		        		(red1+3*red2)/4,
		        		(green1+3*green2)/4,
		        		(blue1+3*blue2)/4);
		        Color[] gradient = {transition_25, transition_50, transition_75, BlockGroup.BIG_BOMB};
		        this.gradient = gradient;
	        }
	        for (int n=0; n<gradient.length; n++) {
		        for (int i=0; i<x_values.size(); i++) {
		        	int x = x_values.get(i);
		        	int y = y_values.get(i);
		        	blocks[y][x]     = gradient[n];
	        		blocks[y+1][x]   = blocks[y][x];
	        		blocks[y+1][x+1] = blocks[y][x];
	        		blocks[y][x+1]   = blocks[y][x];
		        }
		        try {
					Thread.sleep(50);
				} catch (InterruptedException e) {}
		        parent.repaint();
	        }
	        if (made) try {
				Thread.sleep(450);
			} catch (InterruptedException e) {}
    	}
    }

    /**
     * A complicated, long method that checks for full lines and makes bombs explode if they are in a full line.
     * 
     * @see #hasFullLines
     * 
     * @param scoreClass the class that keeps track of the store.
     * @param linesFull keeps track of the number of cleared lines (used for chains)
     */
    public void activateFullLines(Score scoreClass, int chained) {
        boolean repaint = false;

        List<Bomb> bombsActivated = new ArrayList<Bomb>();
        int linesFull = 0;
        // Remove full lines
        for (int y = rows - 1; y >= 0; y--) {
            if (isLineFull(y)) {
            	// if an orange block is in the line, add it to the list of bombs
            	linesFull++; // this is used for calculation the vertical bomb radius (chains!)
            	int x = 0;
            	while (x < cols) {
            		if (blocks[y][x].equals(BlockGroup.SMALL_BOMB)) {
            			bombsActivated.add(new SmallBomb(x, y));
            		}
            		if (blocks[y][x].equals(BlockGroup.BIG_BOMB)) {
            			BigBomb b = new BigBomb(x, y);
            			if (!bombsActivated.contains(b)) {
            				bombsActivated.add(b);
            			}
            		}
            		x++;
            	}
            }
        }
        
        // Bomb radius is defined here.
        int bombradiusX = 3; // This radius * 2 + 1 will be the number of blocks destroyed horizontally.
        int bombradiusY = Math.min(chained + linesFull - 1, 3); // This radius * 2 + 1 will be the number of blocks destroyed vertically. I have it max out at 3 (a square 7x7 explosion.)

        while (bombsActivated.size() > 0) {
			repaint = true;
        	List<Bomb> newList = activateBombs(bombsActivated, bombradiusX, bombradiusY, scoreClass); // will turn them gray
        	bombsActivated = newList;
        	// First repaint - shows blocks as gray
        	parent.repaint();
            try {
				Thread.sleep(250);
			} catch (InterruptedException e) {}
            // Make their color null.
            for (int j = 0; j< rows; j++) for (int k = 0; k < cols; k++) {
            	if (EXPLOSION_COLOR.equals(blocks[j][k])) {
            		blocks[j][k] = null;
            	}
            }
            // Redraw again.
            parent.repaint();
        }
        
        // Repaint if necessary
        if (repaint) {

            try {
				Thread.sleep(250);
			} catch (InterruptedException e) {}
            // Now we have to process which blocks will end up falling. To do this,
            // let's put the blocks in groups. For each block, we will:
            // 1. check to see if it is in any of the groups
            // 2. if it is not, make a group for it, then scan in all four directions to add blocks to its group
            // This is inefficient, but that's OK because it's a game.
            List<List<Point>> groups = new ArrayList<List<Point>>();
            for (int j = 0; j< rows; j++) for (int k = 0; k < cols; k++) {
            	if (blocks[j][k] != null) {
	            	boolean inGroup = false;
	            	for (int i = 0; i < groups.size(); i++) {
	            		if (groups.get(i).contains(new Point(k, j))) {
	            			inGroup = true;
	            		}
	            	}
	            	if (!inGroup) {
	            		// Make a new group
	            		List<Point> newGroup = new ArrayList<Point>();
	            		groups.add(newGroup);
	            		// Call the method to do this. newGroup will be filled up
	            		getConnectedBlocks(new Point(k, j), newGroup);
	            	}
            	}
            }
            
            dropFloatingBlocks(groups);
            
            parent.repaint();
            
            chained++;
            activateFullLines(scoreClass, chained); // we need to call this again, in case there's a chain
        }
        makeBigBombs();
    }
    
    private List<Bomb> activateBombs(List<Bomb> bombsActivated, int bombradiusX, int bombradiusY, Score scoreClass) {
    	int score = 0;
    	List<Bomb> toActivate = new ArrayList<Bomb>();
        // For each activated bomb and surrounding blocks, turn them dark gray temporarily.
        for (int i=0; i<bombsActivated.size(); i++) {
        	int x = (int)bombsActivated.get(i).getX();
        	int y = (int)bombsActivated.get(i).getY();
        	int yRad = bombradiusY;
        	int xRad = bombradiusX;
        	if (bombsActivated.get(i) instanceof BigBomb) {
        		yRad = 3;
        		xRad = 4;
        	}
        	for (int j=y-yRad; j<=y+yRad; j++) {
        		for (int k=x-xRad; k<=x+xRad; k++) {
    			
	        		if ((j >= 0) && (j<rows) && (k >= 0) && (k<cols)) {
	        			if (blocks[j][k] != null) {
	        				if (!EXPLOSION_COLOR.equals(blocks[j][k])) {
	            				score += 10;
	        				}
	            			if (BlockGroup.SMALL_BOMB.equals(blocks[j][k])) {
	            				Bomb b = new SmallBomb(k, j);
	            				if (!bombsActivated.contains(b)) {
	            					toActivate.add(b);
	            				}
	            			} else if (BlockGroup.BIG_BOMB.equals(blocks[j][k])) {
	                			Bomb b = new BigBomb(k, j);
	                			if (!bombsActivated.contains(b)) {
	                				toActivate.add(b);
	                			}
	                		}
	        			}
	        			blocks[j][k] = EXPLOSION_COLOR;
	        		}
        		}
        	}
        }
        scoreClass.addScore(score);
        return toActivate;
    }
    
    /**
     * Updated in v1.1.
     */
    private void dropFloatingBlocks(List<List<Point>> groups) {
    	boolean doneDropping = false;
        while (!doneDropping) {
        	doneDropping = true; // might be changed to false later
        	try {
				Thread.sleep(125);
			} catch (InterruptedException e) {}
			for (int i=0; i<groups.size(); i++) {
				System.out.println(i);
				// Do this for each group. We only loop through groups once in each while loop, but we loop through the Bombs several times.
				List<Point> group = groups.get(i);
				Color[] oldColors = new Color[group.size()];
				for (int j=0; j<group.size(); j++) {
					Point p = group.get(j);
					int x = (int)p.getX();
					int y = (int)p.getY();
					oldColors[j] = blocks[y][x]; // store the old color temporarily
					blocks[y][x] = null; // unpaint the block
					y++;
					p.move(x, y); // this changes the block in the group
				}
				boolean moveBackUp = false;
				for (int j=0; (j<group.size() && !moveBackUp); j++) {
					Point p = group.get(j);
					int x = (int)p.getX();
					int y = (int)p.getY();
					if ((y >= rows) || (blocks[y][x] != null)) {
						moveBackUp = true;
					}
				}
				if (moveBackUp) {
					for (int j=0; j<group.size(); j++) {
						Point p = group.get(j);
						int x = (int)p.getX();
						int y = (int)p.getY();
						y--;
						p.move(x, y);
					}
				} else {
					doneDropping = false;
				}
				// Put the blocks back on the board
				for (int j=0; j<group.size(); j++) {
					Point p = group.get(j);
					int x = (int)p.getX();
					int y = (int)p.getY();
					blocks[y][x] = oldColors[j];
				}
			}
			parent.repaint();
        }
    }
    
    private boolean shouldBeColored(int y, int x, List<Point> group) {
    	return (
    			(blocks[y][x] != null) &&
    			(!group.contains(new Point(x, y)))
    					);
    }

    private void getConnectedBlocks(Point p, List<Point> group) {
    	// This is essentially a flood-fill algorithm.
    	Queue<Point> toBeAdded = new LinkedList<Point>();
    	int py = (int)p.getY();
    	int px = (int)p.getX();
    	if (shouldBeColored(py, px, group)) {
    		toBeAdded.add(p);
    		Point c;
    		while (!toBeAdded.isEmpty()) {
    			c = toBeAdded.poll();
    			int cx = (int)c.getX();
    			int cy = (int)c.getY();
    			if (shouldBeColored(cy, cx, group)) {
    				int w = cx; // w for west
    				int e = cx; // e for east
    				while ((w > -1) && (shouldBeColored(cy, w, group))) {
    					w--;
    				}
    				while ((e < cols) && (shouldBeColored(cy, e, group))) {
    					e++;
    				}
    				for (int i=w+1; i<e; i++) {
    					group.add(new Point(i, cy));
    				}
    				for (int i=w+1; i<e; i++) {
    					if ((cy+1 < rows) && (shouldBeColored(cy+1, i, group))) {
    						toBeAdded.add(new Point(i, cy+1));
    					}
    					if ((cy-1 > -1) && (shouldBeColored(cy-1, i, group))) {
    						toBeAdded.add(new Point(i, cy-1));
    					}
    				}
    			}
    		}
    	}
    }

}
