package us.lakora.bombdrop;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;

public class LevelLoader extends Thread {

	public static Color SMALL_BOMB_PNG = new Color(255, 213, 0);
	public static Color BGCOLOR = Color.black;

	private PlayingField parent;
	private int level;
	private int defaultRows;
	private int defaultCols;
	private Score score;

	public LevelLoader(PlayingField parent, int level, int defaultRows, int defaultCols, Score score) {
		this.parent = parent;
		this.level = level;
		this.defaultRows = defaultRows;
		this.defaultCols = defaultCols;
		this.score = score;
	}

	public void run() {
		try {
			BufferedImage bi = null;
			URL levelFile = MainWindow.class.getClassLoader().getResource("levels/" + level + ".png");
			if (levelFile == null) {
				throw new IOException("couldn't make URL for level file");
			}
			bi = ImageIO.read(levelFile);
			score.displayDrops(true);
			int rows = bi.getHeight();
			int cols = bi.getWidth();
			Color[][] blocks = new Color[rows][cols];
			BlockGroup group = new BlockGroup(cols);
			BlockGroup next = new BlockGroup(cols);
			parent.setParams(rows, cols, blocks, group, next);

			for (int x=0; x<cols; x++) for (int y=0; y<rows; y++) {
				Color c = new Color(bi.getRGB(x, rows-y-1));
				if (c.equals(BGCOLOR)) {
					blocks[rows-y-1][x] = null;
				} else if (c.equals(SMALL_BOMB_PNG)) {
					blocks[rows-y-1][x] = BlockGroup.SMALL_BOMB;
				} else {
					blocks[rows-y-1][x] = c;
				}
			}
			new ClearThread(blocks, parent, score).makeBigBombs();
		} catch (IOException e) {
			score.displayDrops(false);
			int rows = defaultRows;
			int cols = defaultCols;
			BlockGroup group = new BlockGroup(cols);
			BlockGroup next = new BlockGroup(cols);
			Color[][] blocks = new Color[rows][cols];
			parent.setParams(rows, cols, blocks, group, next);
		}
		parent.dropTimer.start();
	}

}
