package us.lakora.bombdrop;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import static us.lakora.bombdrop.BombdropMenu.TITLE;

public class MainWindow implements ActionListener {

	public static void main(String[] args) {
		new MainWindow();
	}
	
	private Score scoreLabel;
	private PlayingField pf;
	private PreviewPanel pp;
	private JComboBox<String> menu;
	private BombdropMenu menubar;
	
	public MainWindow() {
		JFrame mainFrame = new JFrame(TITLE);

		URL iconURL = MainWindow.class.getClassLoader().getResource("resources/icon.png");
		if (iconURL != null) {
			mainFrame.setIconImage(new ImageIcon(iconURL).getImage());
		}
		
		mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		menubar = new BombdropMenu();
		mainFrame.setJMenuBar(menubar);
		
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainFrame.setContentPane(mainPanel);
		
		scoreLabel = new Score();
		
		JButton pause = new JButton("Pause");
		pause.addActionListener(this);
		JButton restart = new JButton("Restart Level");
		restart.addActionListener(this);
		
		JPanel levelChangePanel = new JPanel();
		levelChangePanel.setLayout(new BorderLayout());
		levelChangePanel.add(new JLabel("Select level:"));
		menu = new JComboBox<String>();
		menu.addItem("Endless");
		levelChangePanel.add(menu);
		populateLevelMenu();

		menu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				loadLevel();
			}
		});
		
		JPanel rightPanel = new JPanel();
		rightPanel.setBorder(BorderFactory.createEmptyBorder(0, 8, 0, 8));
		rightPanel.setLayout(new GridLayout(3, 1));
		
		JPanel rightPanelN1 = new JPanel();
		rightPanelN1.setLayout(new BoxLayout(rightPanelN1, BoxLayout.Y_AXIS));
		rightPanel.add(rightPanelN1); // first section
		
		rightPanelN1.add(scoreLabel);

		pp = new PreviewPanel();
		pp.setPreferredSize(new Dimension(32, 94));
//		pp.addMouseListener(new MouseAdapter() {
//
//			@Override
//			public void mouseClicked(MouseEvent arg0) {
//				levelChangePanel.
//			}
//			
//		});
		rightPanelN1.add(pp);
		
		JPanel rightPanelN2 = new JPanel();
		rightPanelN2.setLayout(new GridLayout(2, 1));
		rightPanel.add(rightPanelN2); // second section
		
		JPanel rightPanelN21 = new JPanel();
		rightPanelN21.setBorder(BorderFactory.createEmptyBorder(8, 0, 0, 0));
		JLabel rightPanelN22 = new JLabel();
		rightPanelN22.setFocusable(true);
		rightPanelN21.setLayout(new BoxLayout(rightPanelN21, BoxLayout.Y_AXIS));
		rightPanelN2.add(rightPanelN21);
		rightPanelN2.add(rightPanelN22);

		pause.setAlignmentX(Component.CENTER_ALIGNMENT);
		rightPanelN21.add(pause);
		restart.setAlignmentX(Component.CENTER_ALIGNMENT);
		rightPanelN21.add(restart);
		
		JPanel rightPanelN3 = new JPanel();
		rightPanelN3.setLayout(new BoxLayout(rightPanelN3, BoxLayout.Y_AXIS));
		rightPanel.add(rightPanelN3);
		
		levelChangePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		levelChangePanel.setMaximumSize(new Dimension(256, 32));
		JLabel levelChangeLabel = new JLabel("Select a level:");
		levelChangeLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		rightPanelN3.add(levelChangeLabel);
		rightPanelN3.add(levelChangePanel);
		
		pf = new PlayingField(0, scoreLabel, menu, pp);
		loadLevel();
		mainPanel.add(pf, BorderLayout.CENTER);
		mainPanel.add(rightPanel, BorderLayout.EAST);
		
		menubar.setPlayingField(pf);
		
		mainFrame.pack();
		mainFrame.setVisible(true);
	}

	// For the pause button
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equals("Pause")) {
			pf.togglePause();			
		} else {
			scoreLabel.reset();
			pf.reset();
		}
	}
	
	private void loadLevel() {
		String s = (String)menu.getSelectedItem();
		int level;
		if (s.equals("Endless")) {
			level = 0;
		} else {
			level = Integer.parseInt(s);
		}
		pf.setLevel(level);
		pf.requestFocusInWindow();
	}
	
	private void populateLevelMenu() {
		int i=1;
		while (MainWindow.class.getClassLoader().getResource("levels/" + i + ".png") != null) {
			menu.addItem(i+"");
			i++;
		}
	}

}
