package us.lakora.bombdrop;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.KeyEventDispatcher;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.Timer;

public class PlayingField extends JPanel implements KeyEventDispatcher {
	
	private static final long serialVersionUID = 1L;

	private Color[][] blocks;
	private int rows;
	private int cols;
	private BlockGroup group;
	private BlockGroup next;
	private PreviewPanel previewPanel;
	
	public static final Color BGCOLOR = Color.black;	
	public static final boolean DEBUG = false;
	public Timer dropTimer;
	private Timer fastDropTimer;
	private Score score;
	private int level;
	private JComboBox<String> menu; // updated when level changes
	
	private boolean paused;
	private boolean hardpaused; // for about dialog. When hardpaused is true, paused should be true as well
	private boolean gameover;
	public boolean displayLevelClear;
	
	public PlayingField(int level, Score score, JComboBox<String> menu, PreviewPanel previewPanel) {
		Toolkit.getDefaultToolkit().addAWTEventListener(new RepeatingReleasedEventsFixer(this), AWTEvent.KEY_EVENT_MASK);
		
		this.score = score;
		this.menu = menu;
		this.previewPanel = previewPanel;
		paused = false;
		hardpaused = false;
		gameover = false;
		dropTimer = new Timer((int)(Score.DELAYS[0]/.06), dropBlock); // Game Boy ran at 60 frames/sec
		fastDropTimer = new Timer(50, dropBlock);
//		KeyboardFocusManager kfm = KeyboardFocusManager.getCurrentKeyboardFocusManager();
//		kfm.addKeyEventDispatcher(this);
		
		// temporary - passed to loadLevel, but if an actual level is loaded it will be changed
		cols = 10;
		rows = 20;
		setPreferredSize(new Dimension(cols*24, rows*24));
		
		this.level = level;
		loadLevel(level, rows, cols);
	}
	
	private void loadLevel(int level) {
		loadLevel(level, this.rows, this.cols);
	}
	
    private void loadLevel(int level, int defaultRows, int defaultCols) {
    	dropTimer.stop();
    	LevelLoader ll = new LevelLoader(this, level, defaultRows, defaultCols, score);
    	ll.start();
    }
    
    /**
     * Only used by LevelLoader.
     * @param rows
     * @param cols
     * @param blocks
     * @param group
     * @param next
     */
    public void setParams(int rows, int cols, Color[][] blocks, BlockGroup group, BlockGroup next) {
    	this.rows = rows;
    	this.cols = cols;
    	this.blocks = blocks;
    	this.group = group;
    	setNext(next);
    }
	
    /**
     * Resets to the current level.
     */
	public void reset() {
		dropTimer.stop();
		fastDropTimer.stop();
		
//		group = new BlockGroup(cols);
//		setNext(new BlockGroup(cols));
//		blocks = new Color[rows][cols];
//		repaint();
		loadLevel(level);
		paused = false;
		gameover = false;
		dropTimer.setDelay((int)(Score.DELAYS[0]/.05973));
		
		dropTimer.start();
		repaint();
	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (gameover) {
			g.setColor(new Color(128,64,64));
		} else {
			g.setColor(BGCOLOR);
		}
		g.fillRect(0, 0, this.getWidth(), this.getHeight());
		if (displayLevelClear) {
			g.setColor(Color.white);
			g.setFont(new Font("DejaVu Sans", Font.BOLD, 24));
			g.drawString("Level Clear!", this.getWidth()/2-80, this.getHeight()/2);
		} else if (paused) {
			g.setColor(Color.white);
			g.setFont(new Font("DejaVu Sans", Font.BOLD, 24));
			g.drawString("Paused", this.getWidth()/2-60, this.getHeight()/2);
		}
		for (int y=0; y<rows; y++) for (int x=0; x<cols; x++) {
			if (blocks[y][x] != null) {
				g.setColor(blocks[y][x]);
				g.fillRect(
						(this.getWidth()/cols)*x,
						(this.getHeight()/rows)*y,
						(this.getWidth()/cols),
						(this.getHeight()/rows));
				g.setColor(blocks[y][x].darker());
				g.drawRect(
						(this.getWidth()/cols)*x,
						(this.getHeight()/rows)*y,
						(this.getWidth()/cols),
						(this.getHeight()/rows));
			}
//			g.setColor(Color.darkGray);
//			g.drawRect(
//					(this.getWidth()/cols)*x,
//					(this.getHeight()/rows)*y,
//					(this.getWidth()/cols),
//					(this.getHeight()/rows));
		}
		if (group != null) {
			Color[][] groupBlocks = group.getBlocks();
			for (int y=0; y<groupBlocks.length; y++) for (int x=0; x<groupBlocks[0].length; x++) {
				if (groupBlocks[y][x] != null) {
					g.setColor(groupBlocks[y][x]);
					g.fillRect(
							(this.getWidth()/cols)*(x+group.getCol()),
							(this.getHeight()/rows)*(y+group.getRow()),
							(this.getWidth()/cols),
							(this.getHeight()/rows));
					g.setColor(Color.white);
					g.drawRect(
							(this.getWidth()/cols)*(x+group.getCol()),
							(this.getHeight()/rows)*(y+group.getRow()),
							(this.getWidth()/cols),
							(this.getHeight()/rows));
				}
//				g.setColor(Color.white);
//				g.drawRect(
//						(this.getWidth()/cols)*(x+group.getCol()),
//						(this.getHeight()/rows)*(y+group.getRow()),
//						(this.getWidth()/cols),
//						(this.getHeight()/rows));

			}
		}
	}
	
	public void callButton(int key, boolean pressed) {
		if ((group != null)) {
			if (pressed) {
				if (KeyEvent.VK_ENTER == key && !hardpaused) {
					togglePause();
					repaint();
				} else if (!paused) {
					if (KeyEvent.VK_LEFT == key) {
						group.moveLeft();
						if (checkForConflict()) group.moveRight();
						repaint();
					} else if (KeyEvent.VK_RIGHT == key) {
						group.moveRight();
						if (checkForConflict()) group.moveLeft();
						repaint();
					} else if (KeyEvent.VK_DOWN == key) {
						dropTimer.stop();
						fastDropTimer.start();
					} else if (KeyEvent.VK_X == key) {
						group.rotateRight();
						if (DEBUG) System.out.println("rot right");
						if (checkForConflict()) {
							group.moveRight();
							if (DEBUG) System.out.println("move right to resolve con");
							if (checkForConflict()) {
								group.moveLeft();
								group.moveLeft();
								if (DEBUG) System.out.println("move left to resolve con");
								if (checkForConflict()) {
									group.moveRight();
									group.rotateLeft();
									if (DEBUG) System.out.println("rot left to resolve con");
								}
							}
						}
						repaint();
					} else if (KeyEvent.VK_Z == key) {
						group.rotateLeft();
						if (DEBUG) System.out.println("rot left");
						if (checkForConflict()) {
							group.moveLeft();
							if (DEBUG) System.out.println("move left to resolve con");
							if (checkForConflict()) {
								group.moveRight();
								group.moveRight();
								if (DEBUG) System.out.println("move right to resolve con");
								if (checkForConflict()) {
									group.moveLeft();
									group.rotateRight();
									if (DEBUG) System.out.println("rot right to resolve con");
								}
							}
						}
						repaint();
					}
				}
			} else {
				if (KeyEvent.VK_DOWN == key) {
					fastDropTimer.stop();
					dropTimer.start();
				}
			}
		}
	}
	
	public synchronized boolean dispatchKeyEvent(KeyEvent arg0) {
		int key = arg0.getKeyCode();
		if ((group != null)) {
			if (arg0.getID() == KeyEvent.KEY_PRESSED) {
				if (KeyEvent.VK_ENTER == key && !hardpaused) {
					togglePause();
					repaint();
				} else if (!paused) {
					if (KeyEvent.VK_LEFT == key) {
						group.moveLeft();
						if (checkForConflict()) group.moveRight();
						repaint();
					} else if (KeyEvent.VK_RIGHT == key) {
						group.moveRight();
						if (checkForConflict()) group.moveLeft();
						repaint();
					} else if (KeyEvent.VK_DOWN == key) {
						dropTimer.stop();
						fastDropTimer.start();
					} else if (KeyEvent.VK_X == key) {
						group.rotateRight();
						if (DEBUG) System.out.println("rot right");
						if (checkForConflict()) {
							group.moveRight();
							if (DEBUG) System.out.println("move right to resolve con");
							if (checkForConflict()) {
								group.moveLeft();
								group.moveLeft();
								if (DEBUG) System.out.println("move left to resolve con");
								if (checkForConflict()) {
									group.moveRight();
									group.rotateLeft();
									if (DEBUG) System.out.println("rot left to resolve con");
								}
							}
						}
						repaint();
					} else if (KeyEvent.VK_Z == key) {
						group.rotateLeft();
						if (DEBUG) System.out.println("rot left");
						if (checkForConflict()) {
							group.moveLeft();
							if (DEBUG) System.out.println("move left to resolve con");
							if (checkForConflict()) {
								group.moveRight();
								group.moveRight();
								if (DEBUG) System.out.println("move right to resolve con");
								if (checkForConflict()) {
									group.moveLeft();
									group.rotateRight();
									if (DEBUG) System.out.println("rot right to resolve con");
								}
							}
						}
						repaint();
					}
				}
			} else if (arg0.getID() == KeyEvent.KEY_RELEASED) {
				if (KeyEvent.VK_DOWN == key) {
					fastDropTimer.stop();
					dropTimer.start();
				}
			}
		}
		return false;
	}
	
	private Timer lineClearTimer;

	final PlayingField myself = this;
	
	private AbstractAction dropBlock = new AbstractAction() {
		private static final long serialVersionUID = 1L;

		public synchronized void actionPerformed(ActionEvent arg0) {
			if (checkForConflict()) {
				gameover = true;
				dropTimer.stop();
				fastDropTimer.stop();
				group = null;
				setNext(null);
				repaint();
			} else {
				group.drop();
				if (checkForConflict()) {
					if (!group.waiting) {
						if (DEBUG) System.out.println("making it wait");
						group.waiting = true;
						group.undoDrop();
					} else {
						dropTimer.stop();
						fastDropTimer.stop();
						group.undoDrop();

						if (DEBUG) System.out.println("done waiting");
						lockGroup(); // changes group to null
						if (DEBUG) System.out.println("locked group");
//						// check for full lines, store them in linked list
//						final LinkedList<Integer> fullLines = new LinkedList<Integer>();
//						for (int y=0; y<rows; y++) {
//							boolean emptySpot = false;
//							for (int x=0; (x<cols)&&(!emptySpot); x++) { // stop when end of row reached or empty spot found
//								if (blocks[y][x] == null) {
//									emptySpot = true;
//								}
//							}
//							if (!emptySpot) {
//								// found a full row
//								fullLines.add(y);
//								if (DEBUG) System.out.println("line " + y + " full.");
//							}
//						}
//
//						if (DEBUG) System.out.println("checked all lines");
//						// clear lines - use a timer that waits to start, then stops itself the first time
						AbstractAction clearLines = new AbstractAction() {
							private static final long serialVersionUID = 1L;

							public void actionPerformed(ActionEvent e) {
								lineClearTimer.stop();
//								while (!fullLines.isEmpty()) {
//									int toRemove = fullLines.remove();
//									for (int y=toRemove; y>0; y--) for (int x=0; x<cols; x++) {
//										blocks[y][x] = blocks[y-1][x];
//									}
//									for (int x=0; x<cols; x++) {
//										blocks[0][x] = null;
//									}
//									int newScore = score.addLines(1);
//									int newSpeed = (int)(Score.DELAYS[score.getLevel()]/.05973);
//									dropTimer.setDelay(newSpeed);
//								}
//								if (DEBUG) System.out.println("executed clearLines");

								ClearThread ct = new ClearThread(blocks, myself, score);
								ct.start();
							}
						};
						lineClearTimer = new Timer(200, clearLines);
//						if (!fullLines.isEmpty()) {
//							// turn the full lines blue temporarily
//							for (int n=0; n<fullLines.size(); n++) for (int x=0; x<cols; x++) {
//								blocks[fullLines.get(n)][x] = Color.blue;
//							}
//							repaint();
//							
							lineClearTimer.start();
//						} else {
//							group = next; // replace block group
//							setNext(new BlockGroup(blocks[0].length));
//							if (!paused) dropTimer.start();
//						}
					}
				} else {
					if (fastDropTimer.isRunning()) {
						score.addScore(1);
					}
					repaint();
				}
			}
		}
	};
	
	public void regenerate() {
		group = next; // replace block group
		setNext(new BlockGroup(blocks[0].length));
		repaint();
		if (!paused) dropTimer.start(); // start drop timer again
	}
	
	public void setNext(BlockGroup next) {
		this.next = next;
		if (previewPanel != null) previewPanel.setNext(next);
	}
	
	public boolean checkForConflict() {
		Color[][] groupBlocks = group.getBlocks();
		for (int y=0; y<groupBlocks.length; y++) for (int x=0; x<groupBlocks[0].length; x++) {
			if (groupBlocks[y][x] != null) {
				try {
					if (blocks[y+group.getRow()][x+group.getCol()] != null) {
						return true;
					}
				} catch (ArrayIndexOutOfBoundsException e) {
					return true;
				}
			}
		}
		return false;
	}
	
	private synchronized void lockGroup() {
		Color[][] groupBlocks = group.getBlocks();
		for (int y=0; y<groupBlocks.length; y++) for (int x=0; x<groupBlocks[0].length; x++) {
			Color c = groupBlocks[y][x];
			if (c != null) {
				blocks[y+group.getRow()][x+group.getCol()] = c;
			}
		}
		score.addDrop();
		group = null;
		repaint();
	}
	
	/**
	 * used for pause button
	 */
	public void togglePause() {
		if (!paused) {
			paused = true;
			dropTimer.stop();
			fastDropTimer.stop();
		} else {
			paused = false;
			dropTimer.start();
			fastDropTimer.stop();
		}
	}
	
	/**
	 * used when opening the about dialog. sets both paused and hardpaused, each way.
	 */
	public void hardPause(boolean shouldBePaused) {
		if (!hardpaused && shouldBePaused) {
			paused = true;
			hardpaused = true;
			dropTimer.stop();
			fastDropTimer.stop();
			repaint();
		} else if (hardpaused && !shouldBePaused) {
			paused = false;
			hardpaused = false;
			dropTimer.start();
			fastDropTimer.stop();
			repaint();
		}
	}
	
	public int getLevel() {
		return level;
	}
	
	public void setLevel(int l) {
		if (menu.getItemCount() > l) {
			level = l;
		} else {
			level = 0;
		}
		menu.setSelectedIndex(level);
		menu.repaint();
		score.reset();
		loadLevel(level, rows, cols);
	}
	
	/**
	 * Advances to the next level.
	 */
	public void advanceLevel() {
		setLevel(level+1);
	}

}
